@@ -2,21 +2,20 @@
 A MathML printer.
 """
 
-from __future__ import print_function, division
+from __future__ import annotations
+from typing import Any
 
-from typing import Any, Dict
-
-from sympy import sympify, S, Mul
-from sympy.core.compatibility import default_sort_key
-from sympy.core.function import _coeff_isneg
+from sympy.core.mul import Mul
+from sympy.core.singleton import S
+from sympy.core.sorting import default_sort_key
+from sympy.core.sympify import sympify
 from sympy.printing.conventions import split_super_sub, requires_partial
 from sympy.printing.precedence import \
     precedence_traditional, PRECEDENCE, PRECEDENCE_TRADITIONAL
 from sympy.printing.pretty.pretty_symbology import greek_unicode
-from sympy.printing.printer import Printer
+from sympy.printing.printer import Printer, print_function
 
-import mpmath.libmp as mlib
-from mpmath.libmp import prec_to_dps
+from mpmath.libmp import prec_to_dps, repr_dps, to_str as mlib_to_str
 
 
 class MathMLPrinterBase(Printer):
@@ -24,7 +23,7 @@ class MathMLPrinterBase(Printer):
     MathMLPresentationPrinter.
     """
 
-    _default_settings = {
+    _default_settings: dict[str, Any] = {
         "order": None,
         "encoding": "utf-8",
         "fold_frac_powers": False,
@@ -39,7 +38,7 @@ class MathMLPrinterBase(Printer):
         "root_notation": True,
         "symbol_names": {},
         "mul_symbol_mathml_numbers": '&#xB7;',
-    }  # type: Dict[str, Any]
+    }
 
     def __init__(self, settings=None):
         Printer.__init__(self, settings)
@@ -54,7 +53,7 @@ class MathMLPrinterBase(Printer):
         class RawText(Text):
             def writexml(self, writer, indent='', addindent='', newl=''):
                 if self.data:
-                    writer.write(u'{}{}{}'.format(indent, self.data, newl))
+                    writer.write('{}{}{}'.format(indent, self.data, newl))
 
         def createRawTextNode(data):
             r = RawText()
@@ -74,57 +73,6 @@ class MathMLPrinterBase(Printer):
         res = xmlbstr.decode()
         return res
 
-    def apply_patch(self):
-        # Applying the patch of xml.dom.minidom bug
-        # Date: 2011-11-18
-        # Description: http://ronrothman.com/public/leftbraned/xml-dom-minidom\
-        #                   -toprettyxml-and-silly-whitespace/#best-solution
-        # Issue: http://bugs.python.org/issue4147
-        # Patch: http://hg.python.org/cpython/rev/7262f8f276ff/
-
-        from xml.dom.minidom import Element, Text, Node, _write_data
-
-        def writexml(self, writer, indent="", addindent="", newl=""):
-            # indent = current indentation
-            # addindent = indentation to add to higher levels
-            # newl = newline string
-            writer.write(indent + "<" + self.tagName)
-
-            attrs = self._get_attributes()
-            a_names = list(attrs.keys())
-            a_names.sort()
-
-            for a_name in a_names:
-                writer.write(" %s=\"" % a_name)
-                _write_data(writer, attrs[a_name].value)
-                writer.write("\"")
-            if self.childNodes:
-                writer.write(">")
-                if (len(self.childNodes) == 1 and
-                        self.childNodes[0].nodeType == Node.TEXT_NODE):
-                    self.childNodes[0].writexml(writer, '', '', '')
-                else:
-                    writer.write(newl)
-                    for node in self.childNodes:
-                        node.writexml(
-                            writer, indent + addindent, addindent, newl)
-                    writer.write(indent)
-                writer.write("</%s>%s" % (self.tagName, newl))
-            else:
-                writer.write("/>%s" % (newl))
-        self._Element_writexml_old = Element.writexml
-        Element.writexml = writexml
-
-        def writexml(self, writer, indent="", addindent="", newl=""):
-            _write_data(writer, "%s%s%s" % (indent, self.data, newl))
-        self._Text_writexml_old = Text.writexml
-        Text.writexml = writexml
-
-    def restore_patch(self):
-        from xml.dom.minidom import Element, Text
-        Element.writexml = self._Element_writexml_old
-        Text.writexml = self._Text_writexml_old
-
 
 class MathMLContentPrinter(MathMLPrinterBase):
     """Prints an expression to the Content MathML markup language.
@@ -201,7 +149,7 @@ class MathMLContentPrinter(MathMLPrinterBase):
 
     def _print_Mul(self, expr):
 
-        if _coeff_isneg(expr):
+        if expr.could_extract_minus_sign():
             x = self.dom.createElement('apply')
             x.appendChild(self.dom.createElement('minus'))
             x.appendChild(self._print_Mul(-expr))
@@ -239,7 +187,7 @@ class MathMLContentPrinter(MathMLPrinterBase):
         lastProcessed = self._print(args[0])
         plusNodes = []
         for arg in args[1:]:
-            if _coeff_isneg(arg):
+            if arg.could_extract_minus_sign():
                 # use minus
                 x = self.dom.createElement('apply')
                 x.appendChild(self.dom.createElement('minus'))
@@ -332,9 +280,9 @@ class MathMLContentPrinter(MathMLPrinterBase):
 
     def _print_GoldenRatio(self, e):
         """We use unicode #x3c6 for Greek letter phi as defined here
-        http://www.w3.org/2003/entities/2007doc/isogrk1.html"""
+        https://www.w3.org/2003/entities/2007doc/isogrk1.html"""
         x = self.dom.createElement('cn')
-        x.appendChild(self.dom.createTextNode(u"\N{GREEK SMALL LETTER PHI}"))
+        x.appendChild(self.dom.createTextNode("\N{GREEK SMALL LETTER PHI}"))
         return x
 
     def _print_Exp1(self, e):
@@ -466,9 +414,9 @@ class MathMLContentPrinter(MathMLPrinterBase):
             x.appendChild(self.dom.createElement('root'))
             if e.exp.q != 2:
                 xmldeg = self.dom.createElement('degree')
-                xmlci = self.dom.createElement('ci')
-                xmlci.appendChild(self.dom.createTextNode(str(e.exp.q)))
-                xmldeg.appendChild(xmlci)
+                xmlcn = self.dom.createElement('cn')
+                xmlcn.appendChild(self.dom.createTextNode(str(e.exp.q)))
+                xmldeg.appendChild(xmlcn)
                 x.appendChild(xmldeg)
             x.appendChild(self._print(e.base))
             return x
@@ -485,6 +433,12 @@ class MathMLContentPrinter(MathMLPrinterBase):
         x.appendChild(self.dom.createTextNode(str(e)))
         return x
 
+    def _print_Float(self, e):
+        x = self.dom.createElement(self.mathml_tag(e))
+        repr_e = mlib_to_str(e._mpf_, repr_dps(e._prec))
+        x.appendChild(self.dom.createTextNode(repr_e))
+        return x
+
     def _print_Derivative(self, e):
         x = self.dom.createElement('apply')
         diff_symbol = self.mathml_tag(e)
@@ -534,7 +488,7 @@ class MathMLContentPrinter(MathMLPrinterBase):
 
     def _print_list(self, seq):
         """MathML reference for the <list> element:
-        http://www.w3.org/TR/MathML2/chapter4.html#contm.list"""
+        https://www.w3.org/TR/MathML2/chapter4.html#contm.list"""
         dom_element = self.dom.createElement('list')
         for item in seq:
             dom_element.appendChild(self._print(item))
@@ -569,6 +523,17 @@ class MathMLContentPrinter(MathMLPrinterBase):
             x.appendChild(self._print(arg))
         return x
 
+    def _print_Lambda(self, e):
+        # MathML reference for the lambda element:
+        # https://www.w3.org/TR/MathML2/chapter4.html#id.4.2.1.7
+        x = self.dom.createElement(self.mathml_tag(e))
+        for arg in e.signature:
+            x_1 = self.dom.createElement('bvar')
+            x_1.appendChild(self._print(arg))
+            x.appendChild(x_1)
+        x.appendChild(self._print(e.expr))
+        return x
+
     # XXX Symmetric difference is not supported for MathML content printers.
 
 
@@ -629,6 +594,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
             'mathieuc': 'C',
             'mathieusprime': 'S&#x2032;',
             'mathieucprime': 'C&#x2032;',
+            'Lambda': 'lambda',
         }
 
         def mul_symbol_selection():
@@ -701,7 +667,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
                     mrow.appendChild(y)
             return mrow
         mrow = self.dom.createElement('mrow')
-        if _coeff_isneg(expr):
+        if expr.could_extract_minus_sign():
             x = self.dom.createElement('mo')
             x.appendChild(self.dom.createTextNode('-'))
             mrow.appendChild(x)
@@ -716,7 +682,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
         args = self._as_ordered_terms(expr, order=order)
         mrow.appendChild(self._print(args[0]))
         for arg in args[1:]:
-            if _coeff_isneg(arg):
+            if arg.could_extract_minus_sign():
                 # use minus
                 x = self.dom.createElement('mo')
                 x.appendChild(self.dom.createTextNode('-'))
@@ -1098,8 +1064,8 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
 
     def _print_AccumulationBounds(self, i):
         brac = self.dom.createElement('mfenced')
-        brac.setAttribute('close', u'\u27e9')
-        brac.setAttribute('open', u'\u27e8')
+        brac.setAttribute('close', '\u27e9')
+        brac.setAttribute('open', '\u27e8')
         brac.appendChild(self._print(i.min))
         brac.appendChild(self._print(i.max))
         return brac
@@ -1169,7 +1135,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
     def _print_Float(self, expr):
         # Based off of that in StrPrinter
         dps = prec_to_dps(expr._prec)
-        str_real = mlib.to_str(expr._mpf_, dps, strip_zeros=True)
+        str_real = mlib_to_str(expr._mpf_, dps, strip_zeros=True)
 
         # Must always have a mul symbol (as 2.5 10^{20} just looks odd)
         # thus we use the number separator
@@ -1452,7 +1418,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
         return mi
 
     def _print_Range(self, s):
-        dots = u"\u2026"
+        dots = "\u2026"
         brac = self.dom.createElement('mfenced')
         brac.setAttribute('close', '}')
         brac.setAttribute('open', '{')
@@ -1657,8 +1623,8 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
         power = expr.args[2]
         sup = self.dom.createElement('msup')
         brac = self.dom.createElement('mfenced')
-        brac.setAttribute('close', u'\u27e9')
-        brac.setAttribute('open', u'\u27e8')
+        brac.setAttribute('close', '\u27e9')
+        brac.setAttribute('open', '\u27e8')
         brac.appendChild(self._print(shift))
         sup.appendChild(brac)
         sup.appendChild(self._print(power))
@@ -1775,7 +1741,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
         return sup
 
     def _print_MatMul(self, expr):
-        from sympy import MatMul
+        from sympy.matrices.expressions.matmul import MatMul
 
         x = self.dom.createElement('mrow')
         args = expr.args
@@ -1784,7 +1750,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
         else:
             args = list(args)
 
-        if isinstance(expr, MatMul) and _coeff_isneg(expr):
+        if isinstance(expr, MatMul) and expr.could_extract_minus_sign():
             if args[0] == -1:
                 args = args[1:]
             else:
@@ -1847,8 +1813,8 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
     def _print_floor(self, e):
         mrow = self.dom.createElement('mrow')
         x = self.dom.createElement('mfenced')
-        x.setAttribute('close', u'\u230B')
-        x.setAttribute('open', u'\u230A')
+        x.setAttribute('close', '\u230B')
+        x.setAttribute('open', '\u230A')
         x.appendChild(self._print(e.args[0]))
         mrow.appendChild(x)
         return mrow
@@ -1856,8 +1822,8 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
     def _print_ceiling(self, e):
         mrow = self.dom.createElement('mrow')
         x = self.dom.createElement('mfenced')
-        x.setAttribute('close', u'\u2309')
-        x.setAttribute('open', u'\u2308')
+        x.setAttribute('close', '\u2309')
+        x.setAttribute('open', '\u2308')
         x.appendChild(self._print(e.args[0]))
         mrow.appendChild(x)
         return mrow
@@ -2069,6 +2035,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
         return x
 
 
+@print_function(MathMLPrinterBase)
 def mathml(expr, printer='content', **settings):
     """Returns the MathML representation of expr. If printer is presentation
     then prints Presentation MathML else prints content MathML.
@@ -2088,7 +2055,7 @@ def print_mathml(expr, printer='content', **settings):
     ========
 
     >>> ##
-    >>> from sympy.printing.mathml import print_mathml
+    >>> from sympy import print_mathml
     >>> from sympy.abc import x
     >>> print_mathml(x+1) #doctest: +NORMALIZE_WHITESPACE
     <apply>
@@ -2109,9 +2076,7 @@ def print_mathml(expr, printer='content', **settings):
     else:
         s = MathMLContentPrinter(settings)
     xml = s._print(sympify(expr))
-    s.apply_patch()
     pretty_xml = xml.toprettyxml()
-    s.restore_patch()
 
     print(pretty_xml)
 
